// Sketch_Text Shader
//
// Copyright Outerspace Software, all rights reserved

string description = "Use this effect with a text layer with the bevel styles LIGHT DISCS, LIGHT BULBS, COMB, TUBULAR ROUND, TUBULAR SQUARE and WIREFRAME.";

//some font styles abuse the tangent field for special info:
//tangent.x:	bulb/bulbs
//tangent.y:	contour
//tangent.z:	contours

float4x4	World;
float4x4	ViewProjection;
float4x4	WorldViewProjection;

float4		DiffuseCol;

float4		ImageResolution0;
float4 		UVBounds0;
float4		CharacterIndex;

float cPI=3.14159;

bool optimization=true;

float4 Prop0
<
string UIName="Writer";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,0,0);
int UISliders=2;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop1
<
string UIName="Smooth";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0.05,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0.05,0,0,0);

float4 Prop2
<
string UIName="Colour Tip";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,1,0);
int UISliders=3;
float UIScale=255;
bool UIInteger=true;
int UIWidget=2;
> = float4(1,1,0,1);

float4 Prop3
<
string UIName="Brightness Gradient Side";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0.75,0.25,0);
int UISliders=2;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0.75,0.25,0,1);

float4 Prop4
<
string UIName="Shadow Colour";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0.5,0.5,0.5);
int UISliders=3;
float UIScale=255;
bool UIInteger=true;
int UIWidget=2;
> = float4(0.5,0.5,0.5,1);

float4 Prop5
<
string UIName="Shadow Position";
float UIMin=-1;
float UIMax=1;
float3 UIDefault=float3(0.5,0.5,0);
int UISliders=2;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0.5,0.5,0,0);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
	float3 tangent:		TANGENT;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float3 bulbinfo: 	TEXCOORD0;
	float  depth:		TEXCOORD1;	
};

struct PS_OUTPUT{
	float4 color:		COLOR;
};

float GetAlpha(float2 inWriter, float3 inBulbinfo){

	float p = CharacterIndex.x / CharacterIndex.y;
	p += (inBulbinfo.y + inBulbinfo.x) / (CharacterIndex.y * inBulbinfo.z);

	//smooth
	float s=Prop1.x;
	if(CharacterIndex.y>0) s/=CharacterIndex.y;
	if(s<0) s=0;
	if(s>1) s=1;
	
	inWriter.x = -s + inWriter.x*(1+2*s);
	inWriter.y = -s + inWriter.y*(1+2*s);

	if(p>inWriter.x+s) return 0;
	if(p<inWriter.y-s) return 0;

	if(s>0){
		if(p>inWriter.x) return 1-(p-inWriter.x)/s;
		if(p<inWriter.y) return 1-(inWriter.y-p)/s;
	}

	return 1;
}

float4 GetColour(float3 inBulbinfo){

	float a=GetAlpha(Prop0.xy,inBulbinfo);

	float f=1;

	if(CharacterIndex.y>0) f=CharacterIndex.x / CharacterIndex.y;

	if(CharacterIndex.y>0 && inBulbinfo.z>0) f=f+(inBulbinfo.y / inBulbinfo.z) /  CharacterIndex.y;

	if(f<0) f=0;
	if(f>1) f=1;

	float4 c=DiffuseCol*(1-f) + Prop2*f;

	if(a<=0) return float4(0,0,0,0);

	c.w=a;

	return c;
}

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;


	float z=input.position.z;
	z=(z+1)/2;
	output.depth=z;


	float4 Pos=input.position;
	Pos.z*=10; //op deze manier is de tekst even dik als sketch_voxel

	Pos=mul(Pos, World);
	output.position = mul(Pos, ViewProjection);

	output.bulbinfo = input.tangent.xyz;
	return output;
}

//inZ loopt van 0(voorkant) tot 1(achterkant)

float GetSideBrightness(float inZ, float inB0, float inB1){

	const float d1=0.1; //voorkant constante kleur		
	const float d2=0.2; //verloop naar inB0

	//Gradient side
	if(inZ>d1 && inZ<d2){
		float f=(inZ-d1)/(d2-d1);
		f=1 + f*(inB0-1);
		return f;
	}

	if(inZ>d2){
		float f=(inZ-d2)/(1.0-d2);
		f=inB0 + f*(inB1-inB0);
		return f;
	}

	return 1;
}

PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;

	float4 c = GetColour(input.bulbinfo);

	c.xyz *= GetSideBrightness(input.depth,Prop3.x,Prop3.y);

	c.w *= DiffuseCol.w;

	output.color = c;

	return output;
}

VS_OUTPUT VS_SHADOW(VS_INPUT input){
	VS_OUTPUT output;

	float z=input.position.z;
	z=(z+1)/2;
	output.depth=z;


	float4 pos=input.position;
	pos.z*=10; //op deze manier is de tekst even dik als sketch_voxel



	float k=z;
	k*=10;
	pos.x += Prop5.x*k;
	pos.y += Prop5.y*k;
	pos.z = 10;




	pos=mul(pos, World);
	output.position = mul(pos, ViewProjection);

	output.bulbinfo = input.tangent.xyz;
	return output;
}

PS_OUTPUT PS_SHADOW(VS_OUTPUT input){
	PS_OUTPUT output;

	float4 c = float4(0,0,0,1);

	c.w=GetColour(input.bulbinfo).w;

	c.xyz=Prop4.xyz;
	output.color = c;
	return output;
}

technique Shader
{
    //shadows
    pass P0
    { 
	Alphablendenable= TRUE;
	SRCBLEND	= SRCALPHA;
	DESTBLEND	= INVSRCALPHA;

	ALPHATESTENABLE	= TRUE;
	ALPHAREF	= 1;
	ALPHAFUNC	= GREATEREQUAL;

	ZWriteEnable	= FALSE;
	
	VertexShader = compile vs_3_0 VS_SHADOW();
	PixelShader  = compile ps_3_0 PS_SHADOW();
    }

    pass P1
    { 
	Alphablendenable= TRUE;
	SRCBLEND	= SRCALPHA;
	DESTBLEND	= INVSRCALPHA;

	ALPHATESTENABLE	= TRUE;
	ALPHAREF	= 1;
	ALPHAFUNC	= GREATEREQUAL;

	ZWriteEnable	= TRUE;
	
	VertexShader = compile vs_3_0 VS();
	PixelShader  = compile ps_3_0 PS();
    }
}