// Sketch_Volume Effect
//
// Copyright Outerspace Software, all rights reserved

string description = "This effect animates a sketch map as generated by Bixelangelo. The 1st texture must be a sketch map and is used for timing. The 2nd texture is used for the alpha channel, the antialiasing and the colour map. If the FX Colour Map Factor property is set to 0, the 2nd texture can be the same as the 1st texture. The antialiasing quality is best when using a 2nd texture, even when it's the same as the 1st.";

float4x4	WorldViewProjection;
float4		DiffuseCol;
float4 		ImageResolution0;
float4 		ImageResolution1;

const float cR=100;

bool optimization=true;

float4 Prop0
<
string UIName="Writer";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,0,0);
int UISliders=2;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop1
<
string UIName="Smooth";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0.05,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0.05,0,0,0);

float4 Prop2
<
string UIName="Colour Tip";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,1,1);
int UISliders=3;
float UIScale=255;
bool UIInteger=true;
int UIWidget=2;
> = float4(1,1,1,1);

float4 Prop3
<
string UIName="Beam Length";
float UIMin=0;
float UIMax=2;
float3 UIDefault=float3(1,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop4
<
string UIName="Beam Intensity";
float UIMin=0;
float UIMax=2;
float3 UIDefault=float3(1,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop5
<
string UIName="Beam Direction";
float UIMin=-400;
float UIMax=400;
float3 UIDefault=float3(0,0,100);
int UISliders=3;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,100,0);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = LINEAR;
   MagFilter = LINEAR;
   MipFilter = LINEAR;   
   AddressU  = Clamp;
   AddressV  = Clamp;
};

texture MyTexture1;
sampler MyTextureSampler1 = sampler_state
{
   Texture = <MyTexture1>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float3 tex:		TEXCOORD0;
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;

	float4 pos=input.position;
	pos.xyz / pos.w;
	pos.w=1;

	float sx=1;
	float sy=1;
	if(ImageResolution0.x>0){
		sy=1.25;
		sx=(sy*ImageResolution0.x) / ImageResolution0.y;
	}

	pos.x*=sx;
	pos.y*=sy;
	pos.z-=cR;

	float f=-pos.z/(2*cR);	// Achterkant:0, Voorkant:1

	pos.x += (pos.x*Prop5.z*0.01 + Prop5.x)*f*Prop3.x;
	pos.y += (pos.y*Prop5.z*0.01 + Prop5.y)*f*Prop3.x;
	pos.z*=Prop3.x;

	output.position = mul(pos, WorldViewProjection);
	output.tex.x = (input.position.x+cR)/(2*cR);
	output.tex.y = 1-(input.position.y+cR)/(2*cR);
	output.tex.z = 1-f;

	return output;
}

float4 GetColour(float2 inUV, float2 inStage){

	float4 q0=float4(1,1,1,1);
	float4 q1=float4(1,1,1,1);

	if(ImageResolution0.x>0){
		q0=tex2D(MyTextureSampler0,inUV);
		q1=q0;
	}
	if(ImageResolution1.x>0){
		q1 = tex2D(MyTextureSampler1,inUV);
	}

	float p=(q0.x*65536+q0.y*256+q0.z)/65793.0;

	//colour gradient
	float4 c = DiffuseCol*(1-p) + Prop2*p;

	if(q1.w<=0){
		c.w=0;
		return c;
	}

	float Smooth=Prop1.x;

	float t=p;

	float r=1;

	// einde
	float b=-Smooth+inStage.x*(1+2*Smooth);
	if(t>=(b+Smooth)) r=0;
	if(t>b && t<b+Smooth) r=1-(t-b)/Smooth;
	
	// begin
	b=-Smooth+inStage.y*(1+2*Smooth);
	if(t<=b) r=0;
	if(t>b && t<b+Smooth) r*=((t-b)/Smooth);

	r*=q1.w;
	c.w=r;
	return c;
}

float4 FixColour(float4 c){
	c.xyz*=c.w;
	c.w=c.x;
	if(c.y>c.w) c.w=c.y;
	if(c.z>c.w) c.w=c.z;
	return c;
}

PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;

	float4 c=float4(0,0,0,1);

	if(ImageResolution0.x>0){
		c=GetColour(input.tex.xy, Prop0.xy);
	}

//	if(c.w>0.5) c.w=1.0-c.w;

	c.w*=DiffuseCol.w;

	c.w*=0.1;
	c.w*=Prop4.x;

	c.xyz *= input.tex.z;

	output.color=FixColour(c);
	return output;
}

technique Shader
{
	pass P1
	{
		Alphablendenable= TRUE;
		SrcBlend	= ONE;
		DestBlend	= ONE;
		CullMode	= NONE;
		ZWriteEnable	= FALSE;

		VertexShader	= compile vs_3_0 VS();
		PixelShader	= compile ps_3_0 PS();
	}
}


